unit FDFunction;

interface

uses
  System.Generics.Collections, System.Classes, System.SysUtils, System.Variants,
  Data.DB,
  FireDAC.Comp.Client, FireDAC.Stan.Option;

type
  TFDFunction = class
  strict private
    class procedure Connect(AConnection: TFDcustomConnection); static;
  public
    class procedure FreeAndNilQry(var Query: TFDQuery; FieldList: TList<TField>); static;
    class procedure QryOpen(const Query: TFDQuery; FieldList: TList<TField>); static;
    class function NewQry(const AOwner: TComponent = nil): TFDQuery; static;
  end;

implementation

uses
  dmBasic;

class procedure TFDFunction.Connect(AConnection: TFDcustomConnection);
begin
  if not AConnection.Connected then
    AConnection.Connected := True;
end;

{ TFDFunction }

class procedure TFDFunction.FreeAndNilQry(var Query: TFDQuery; FieldList: TList<TField>);
begin
  if not Assigned(Query) then
    Exit;

  FreeAndNil(Query);

  if Assigned(FieldList) then
    FreeAndNil(FieldList);
end;

class function TFDFunction.NewQry(const AOwner: TComponent = nil): TFDQuery;
begin
  if Assigned(AOwner) then
    Result := TFDQuery.Create(AOwner)
  else
    Result := TFDQuery.Create(nil);

  Result.Connection := dtmdBasic.conFbConnection;
  Result.FetchOptions.Mode := fmAll;
  Connect(Result.Connection);
end;

class procedure TFDFunction.QryOpen(const Query: TFDQuery; FieldList: TList<TField>);
var
  Field: TField;
begin
  Assert(Assigned(Query), '  Query is nil  ');
  Assert(Query.SQL.Text <> '', '  Query.SQL.Text is empty  ');

  if not Assigned(Query.Transaction) then
    Query.Transaction := dtmdBasic.fdtrReadOnlyTransaction
  else
  if not Query.Transaction.Active then
    Query.Transaction.StartTransaction;

  if Query.Active then
    Query.Refresh
  else
    Query.Open;

  Query.First;

    if Assigned(FieldList) then
    begin
      if FieldList.Count > 0 then
        FieldList.Clear;

      for Field in Query.Fields do
        FieldList.Add(Field);
    end;
end;

end.
